/*
    This Software (C) Copyright David Goodwin, 2007, 2008.
    
     This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 * MainWindow.java
 *
 * Created on 23 July 2007, 21:07
 *
 */

package assignment;

/* Required packages */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JFileChooser;
import java.util.prefs.*;
import logging.*;
import logging.ILogger.Severity;


/** Program entry point and main window class.
 *
 * @author David
 */
public class MainWindow implements ActionListener, WindowListener  {
    protected JFrame window;        // The main window.
    protected ForceLTSView theView;      // Displays an LTS
    protected LTS currentLTS;       // The currently open LTS.
    protected JFileChooser jfc;     // The file chooser we will use
    protected Menu theMenu;
    protected settings theSettings;
    
    /**
     * Creates a new instance of MainWindow
     */
    public MainWindow(settings ts) {
       
        this.theSettings = ts;
        
        window = new JFrame(globals.getAppNameString());
        window.addWindowListener(this);
        window.setSize(800,600);
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jfc = new JFileChooser();
        if (theSettings.getDialogLocation() != null)
            jfc.setCurrentDirectory(new File(theSettings.getDialogLocation()));

        theMenu = new Menu(this, theSettings);

        window.setLayout(new BorderLayout());
        
        window.setJMenuBar(theMenu);
        
        // Create a new LTS View panel.
        theView = new ForceLTSView(theSettings);
        
        window.add(theView, BorderLayout.CENTER);
        
        JToolBar tb = new JToolBar("Exciting buttons of doom");
        addTBButtons(tb);
        window.add(tb,BorderLayout.PAGE_START);
        
        
        window.setVisible(true);
        
        globals.log.writeLine("Autocreating LTS");
        fileNew();
    }
    
    /** Adds the buttons to the toolbar
     */
    public void addTBButtons(JToolBar tb) {
        JButton button = null;
        
        button = new JButton();
        button.setText("Open");
        button.addActionListener(this);
        tb.add(button);
        
        button = new JButton();
        button.setText("Compose");
        button.addActionListener(this);
        tb.add(button);
        
        button = new JButton();
        button.setText("Reach");
        button.addActionListener(this);
        tb.add(button);
        
        button = new JButton();
        button.setText("Redraw");
        button.addActionListener(this);
        tb.add(button);
        
        button = new JButton();
        button.setText("Iterate");
        button.addActionListener(this);
        tb.add(button);
        
        button = new JButton();
        button.setText("Reset Layout");
        button.addActionListener(this);
        tb.add(button);
    }
    
    /** Handles the toolbar events
     */
    public void actionPerformed(ActionEvent event){
        String s = event.getActionCommand();
        
        if (s == "Open")
            handleMenuEvent("Open...");
        if (s == "Compose")
            handleMenuEvent("Compose with...");
        if (s == "Reach")
            handleMenuEvent("Parallel Reach...");
        if (s == "Parallel Reach...")
            handleMenuEvent("Parallel Reach...");
        if (s == "Redraw")
            handleMenuEvent("Refresh");
        if (s == "Iterate")
            handleMenuEvent("Sim Iterate");
        if (s == "Reset Layout")
            handleMenuEvent(s);
     }
    
    /** Processes a menu click event.
     */
    public void handleMenuEvent(String menuItemPressed) {
        if (menuItemPressed == "New")
            fileNew();
        else if (menuItemPressed == "Open...")
            fileOpen();
        else if (menuItemPressed == "Compare...") {
            // Open the first LTS
            fileOpen();
            LTS firstLTS = currentLTS;
            
            // Open the second LTS
            fileOpen();
            LTS secondLTS = currentLTS;
            
            // reset the current LTS
            fileNew();
            
            // Compare the LTS
            boolean result = LTS.bisimEquals(firstLTS, secondLTS);
            
            // Display the result
            String title = new String("LTS Compare");
            String message = null;
            if (result) message = new String("The two selected LTS are equivalent or equal.");
            else message = new String("The two selected LTS are not equivalent or equal."); 
            
            theView.showMessage(title, message);
        } 
        else if (menuItemPressed == "Save")
            fileSave();
        else if (menuItemPressed == "Save As...")
            fileSaveAs();
        else if (menuItemPressed == "Compose with...")
            editCompose();
        else if (menuItemPressed == "Parallel Reach...")
            editParallelReach();
        else if (menuItemPressed == "Simplify") {
            currentLTS = LTS.simplify(currentLTS);
            theView.setLTS(currentLTS);
            theView.setup();
            theView.renderLTS();
        }
        else if (menuItemPressed == "Exit")
            exit();
        else if (menuItemPressed == "Refresh")
            updateDisplay();
        else if (menuItemPressed == "Sim Iterate")
            iterate();
        else if (menuItemPressed == "Reset Layout")
            resetLayout();
        else if (menuItemPressed.startsWith("Show Colours")) {
            theView.showColors();
        }
        else if (menuItemPressed.startsWith("Show Node Labels")) {
            theSettings.showNodeLabels(!theSettings.showNodeLabels());
            theView.repaint();
        }
        else if (menuItemPressed.startsWith("Show Transition Labels")) {
            theSettings.showTransitionLabels(!theSettings.showTransitionLabels());
            theView.repaint();
        }   
        else if (menuItemPressed.startsWith("Auto-Generate Node Labels")) {
            theSettings.autoGenerateNodeLabels(!theSettings.autoGenerateNodeLabels());
        }
        else if (menuItemPressed.startsWith("Auto-Generate Transition Labels")) {
            theSettings.autoGenerateTransitionLabels(!theSettings.autoGenerateTransitionLabels());
        }
        else if (menuItemPressed == "About") {
            theView.showMessage("About " + globals.getAppNameString(), 
                    "A simple COMP242 assignment that lived too long.\n" +
                    "      (C) Copyright David Goodwin, 2009.\n \n \n" +
                    "Licensed under the GNU GPL v2 or later");            
        }
        else if (menuItemPressed == "Help") {
            theView.showMessage("Help","There isnt any user manual available at this stage. For " +
                    "basic usage information, see the README file that came with this program.");
        }
        else             
            globals.log.writeLine(Severity.Error, "Unknown menu event name \"" + menuItemPressed + "\"");
    }
    
    /** Switches to the specified tool
     */
    public void switchTool(String ToolName) {
        /* Tools for the Viewer/Editor are:
         * 0 - No tool
         * 1 - Select
         * 2 - Set Start Node
         * 3 - Add Node
         * 4 - Add Transition
         * 5 - Remove Node
         * 6 - Remove Transition
         * 7 - Move Node
         * 8 - Deselect currently selected node
         */
        
        if (ToolName == "Select")
            theView.setTool(1);
        else if (ToolName == "Set Start Node")
            theView.setTool(2);
        else if (ToolName == "Add Node")
            theView.setTool(3);
        else if (ToolName == "Add Transition")
            theView.setTool(4);
        else if (ToolName == "Remove Node")
            theView.setTool(5);
        else if (ToolName == "Remove Transition")
            theView.setTool(6);
        else if (ToolName == "Move Node")
            theView.setTool(7);
        else if (ToolName == "Deselect")
            theView.setTool(8);
        else
            theView.setTool(0);
    }
    
    /** This is for future use by a most recently used file menus
     */
    public void handleOpenEvent(String fileName) {
        try {
            theView.showMessage("Loading...", "The LTS is currently loading. This could take a while depending on the number of nodes in the LTS.");
            currentLTS = LTS.load(fileName);
            
            if (currentLTS == null) {
                theView.showMessage("ERROR!","The selected file does not appear to be a valid LTS. Please try again with a different file.");
                return;
            }

            updateTitle(fileName);

            theView.showMessage("Configuring...", "The LTS has been loaded into memory. The LTS Renderer is now being configured. This could take a very long time if the LTS is very large.");

            theSettings.addMRUItem(fileName);
            theMenu.updateMRUList();   
            
            theView.setLTS(currentLTS);
            theView.setup();
        }
        catch (IOException e) {
            globals.log.writeLine(Severity.Error, "File load failed: IO Exception");
        }
        theView.hideMessage();
    }
    
    /** Creates a new LTS.
     */
    private void fileNew() {
        globals.log.writeLine("Creating new LTS...");
        currentLTS = new LTS();
        updateTitle("Untitled");
        theView.setLTS(currentLTS);
        theView.setup();
        //updateDisplay();
    }
    
    /** Composes the current LTS with another. First asks the user to select the
     * file containing the LTS to compose with the current one. Then asks the
     * user to enter the Sync Set.
     */
    private void editCompose() {
        LTS ltsA = currentLTS;
        if (!fileOpen()) {
            // Open operation canceled: die.
            globals.log.writeLine("Compose Canceled by user");
            return;
        }
        currentLTS.setFileName(ltsA.getFileName());
        updateTitle(currentLTS.getFileName());
        LTS ltsB = currentLTS;
        
        // Ok, now we need to get the Sync set.
        String syncSet = JOptionPane.showInputDialog("Enter Sync Set:");
        
        currentLTS = LTS.composeA(ltsA, ltsB, syncSet);
        currentLTS.setFileName(ltsA.getFileName());
        updateTitle(currentLTS.getFileName());
        theView.setLTS(currentLTS);
        theView.setup();
    }
    
    /** Parallel Reach composes the current LTS with another. First asks the 
     * user to select the file containing the LTS to compose with the current 
     * one. Then asks the user to enter the Sync Set.
     */
    private void editParallelReach() {
        LTS ltsA = currentLTS;
        if (!fileOpen()) {
            // Open operation canceled: die.
            globals.log.writeLine("Parallel Reach Canceled by user");
            return;
        }
        currentLTS.setFileName(ltsA.getFileName());
        updateTitle(currentLTS.getFileName());
        LTS ltsB = currentLTS;
        
        // Ok, now we need to get the Sync set.
        String syncSet = JOptionPane.showInputDialog("Enter Sync Set:");
        
        currentLTS = LTS.Parallel_ReachA(ltsA, ltsB, syncSet);
        currentLTS.setFileName(ltsA.getFileName());
        currentLTS.setFileName(ltsA.getFileName());
        updateTitle(currentLTS.getFileName());
        theView.setLTS(currentLTS);
        theView.setup();
    }
    
    /** Opens an LTS from a text file.
     */
    private boolean fileOpen() {
        if (theSettings.getDialogLocation() != null)
            jfc.setCurrentDirectory(new File(theSettings.getDialogLocation()));
        int returnVal = jfc.showOpenDialog(jfc);
        theSettings.setLastDirectory(jfc.getCurrentDirectory().toString());
       
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            /* Load the file */
            File file = jfc.getSelectedFile();
                       
            handleOpenEvent(file.getAbsolutePath());
            
            return true;
            
            /* Update the display */
            //updateDisplay();
        }
        else {
            globals.log.writeLine("File open operation canceled by user.");
            return false;
        }
    }
    
    /** Saves the current LTS.
     */
    private void fileSave() {
        // if it is a new LTS then we have to call saveAs instead.
        if (currentLTS.getFileName() == "") {
            fileSaveAs();
            return;
        }
        try {
            theView.flushChanges();
            currentLTS.save();
        }
        catch (IOException e) {
            globals.log.writeLine("File Save failed: " + e.toString());
        }
    }
    
    /** Saves the current LTS with a different file name.
     */
    private void fileSaveAs() {
        if (theSettings.getDialogLocation() != null)
            jfc.setCurrentDirectory(new File(theSettings.getDialogLocation()));
        int returnVal = jfc.showSaveDialog(jfc);
        theSettings.setLastDirectory(jfc.getCurrentDirectory().toString());
       
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = jfc.getSelectedFile();
            try {
                theView.flushChanges();
                currentLTS.save(file.getAbsolutePath());
                theSettings.addMRUItem(file.getAbsolutePath());
                theMenu.updateMRUList();
                updateTitle(file.getAbsolutePath());
            }
            catch (IOException e) {
                globals.log.writeLine(Severity.Error, "File Save failed: " + e.toString());
            }
        } else {
            globals.log.writeLine("File save operation canceled by user.");
        }
    }
     
    /** Exits the program.
     */
    private void exit() {
        theSettings.saveSettings();
        System.out.println("Application terminated by user.");
        System.exit(0);     /* Terminate Normally */
    }
    
    /** Resets the layout so that it may be rerun if the current layout
     *  isnt ideal.
     */
    private void resetLayout() {
        //window.remove(theView);
        //theView = new ForceLTSView();
        //theView.setLTS(currentLTS);
        //window.add(theView, BorderLayout.CENTER);
        theView.setup();
        theView.repaint();
    }
    
    /** Updates the display (recalculates/redraws the LTS)
     */
    private void updateDisplay() {
        globals.log.writeLine("Updating display...");
        theView.renderLTS();
        globals.log.writeLine("Display updated.");
    }
    
    /** Updates the string displayed in the title bar
     */
    private void updateTitle(String addition) {
        window.setTitle(globals.getAppNameString() + " - [" + addition + "]");
    }
    
    /** Does one iteration of the layout algorithm
     */
    private void iterate() {
        Double val = theView.iterate();
        //updateTitle(val.toString());
    }
    
    /** This is called when the window is closed so we can save settings, etc.
     * 
     * @param arg0 ???
     */
    public void windowClosing(WindowEvent arg0) {
        exit();
    }

    public void windowOpened(WindowEvent arg0) {
    }
    public void windowClosed(WindowEvent arg0) {
    }
    public void windowIconified(WindowEvent arg0) {
    }
    public void windowDeiconified(WindowEvent arg0) {
    }
    public void windowActivated(WindowEvent arg0) {
    }
    public void windowDeactivated(WindowEvent arg0) {
    }
    
    
    /** Program entry point.
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        System.out.println(globals.getAppNameString() + "\n     (C) Copyright David Goodwin, 2009");
        System.out.println("\nProgram Starting...");       
        
        // Load the settings
        settings theSettings = new settings();
        try {
            theSettings.loadSettings(globals.settingsFileName);
        } catch (Exception e) {
            globals.log.writeLine(Severity.Warning, "Settings file not found. " +
                    "Attempting to save default settings to create it.");
            // Create the settings file
            theSettings.saveSettings();
            // And try to load it again
            try {
                theSettings.loadSettings(globals.settingsFileName);
            } catch(Exception ee) {
                ee.printStackTrace();
            }
        }
       
        /* Setup the logger. */
        switch (theSettings.getLoggerType()) {
            case 0:
                globals.log = new DebugConsoleLogger();
                globals.log.writeLine(Severity.Debug, "Using DebugConsoleLogger");
                break;
            case 1:
                globals.log = new ConsoleLogger();
                globals.log.writeLine(Severity.Debug, "Using ConsoleLogger");
                break;
            case 2:
                globals.log = new NullLogger();
                globals.log.writeLine(Severity.Debug, "Using NullLogger");
                break;
            case 3:
                globals.log = new Logger();
                globals.log.writeLine(Severity.Debug, "Using Logger");
                break;
            case 4:
                globals.log = new DebugLogger();
                globals.log.writeLine(Severity.Debug, "Using DebugLogger");
                break;
            default:
                globals.log = new DebugConsoleLogger();
                globals.log.writeLine(Severity.Debug, "Using DebugConsoleLogger");
                break;
        }
        
        try {
            globals.log.open(theSettings.getLogFile());
        } catch(Exception e) {e.printStackTrace();}
        
        MainWindow theApp = new MainWindow(theSettings);
    }
    
}
