/*
    This Software (C) Copyright David Goodwin, 2007, 2008.
    
     This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Menu.java
 *
 * Created on 23 July 2007, 22:38
 *
 */

package assignment;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.prefs.*;
import java.util.HashMap;
import logging.*;
import logging.ILogger.Severity;

/** This class handles the menu used for the main window. It does all the setup
 * and provides some simple event handling.
 *
 * @author David
 */
public class Menu extends JMenuBar {
    
    String[ ] fileItems = new String[ ] { "New", "Input New", "_", "Open...", "Recent Files", "Compare...", "_", "Save", "Save As...", "_", "Exit" };
    String[ ] editItems = new String[ ] { "Compose with...", "Parallel Reach...", "Simplify", "_", "Auto-Generate Node Labels", "Auto-Generate Transition Labels" };
    String[ ] viewItems = new String[ ] { "Refresh", "Sim Iterate", "Reset Layout","_", "-Show Colours", "Show Node Labels", "Show Transition Labels"};
    String[ ] toolItems = new String[ ] { "Select", "Deselect", "Move Node", "Set Start Node", 
        "Add Transition", "Add Node", 
        "Remove Transition", "Remove Node" };
    String[ ] helpItems = new String[ ] { "Help", "_", "About" };
    
    private JMenu recentFiles = null;
    settings theSettings;
    ActionListener openListener = null;
    
    MainWindow mainApp;
    /** Creates a new instance of Menu */
    public Menu(MainWindow theMain, settings prefs) {
        theSettings = prefs;
        //@TODO: Implement shortcut keys
        mainApp = theMain;
        JMenu fileMenu = new JMenu("File");
        JMenu editMenu = new JMenu("Edit");
        JMenu viewMenu = new JMenu("View");
        JMenu toolMenu = new JMenu("Tools");
        JMenu helpMenu = new JMenu("Help");
        
        // Handle the recent files list:
        openListener = new ActionListener(  ) {
        public void actionPerformed(ActionEvent event) {
             mainApp.handleOpenEvent(event.getActionCommand());
             globals.log.writeLine(Severity.Debug, "Menu item [" + event.getActionCommand(  ) + "] was pressed.");
         }
        };
        recentFiles = new JMenu("Recent Files");
        
        // create MRU menus
        updateMRUList();
        
        // A standard menu action listener
        ActionListener printListener = new ActionListener(  ) {
         public void actionPerformed(ActionEvent event) {
             // Filter out any checkable information
             String as = event.getActionCommand();
             if (as.startsWith("+") ||
                     as.startsWith("-")) {
                 as = as.substring(1);
             }
             mainApp.handleMenuEvent(as);
            //System.out.println("Menu item [" + event.getActionCommand(  ) + "] was pressed.");
         }
        };
        
        // An action listener for the tools menu:
        ActionListener toolListener = new ActionListener(  ) {
         public void actionPerformed(ActionEvent event) {
             mainApp.switchTool(event.getActionCommand());
         }
        };
        
        // Assemble the File menu...
        // Woah! This bit has been reverted all the way back from r41 over a
        // year ago!
        for (int i=0; i < fileItems.length; i++) {
            /*if (fileItems[i] == "Recent Files") {
                fileMenu.add(recentFiles);
            }
            else {
                 JMenuItem item = new JMenuItem(fileItems[i]);
                 item.addActionListener(printListener);
                 fileMenu.add(item);
            }*/
            
            if (fileItems[i] == "Recent Files") {
                fileMenu.add(recentFiles);
            }
            else if (fileItems[i].startsWith("_")) {
                // It is a separator
                fileMenu.addSeparator();
            }
            else if (fileItems[i].startsWith("+") || fileItems[i].startsWith("-")) {
                // It is a toggle menu
                JCheckBoxMenuItem item = new JCheckBoxMenuItem(fileItems[i].substring(1));
                item.addActionListener(printListener);
                
                if (fileItems[i].startsWith("+"))
                    item.setState(true);
                else
                    item.setState(false);                
                
                fileMenu.add(item);
            }
            else {
                // Its a normal boring old menu item
                JMenuItem item = new JMenuItem(fileItems[i]);
                item.addActionListener(printListener);
                fileMenu.add(item);
            }
        }              
        
        // And the edit menu...
        if (theSettings.autoGenerateNodeLabels())
            editItems[4] = "+" + editItems[4];
        else
            editItems[4] = "-" + editItems[4];
        if (theSettings.autoGenerateTransitionLabels())
            editItems[5] = "+" + editItems[5];
        else
            editItems[5] = "-" + editItems[5];            
        setupMenu(editItems, editMenu, printListener);
        
        // The view menu...
        HashMap<Integer, KeyStroke> shortcuts = new HashMap<Integer, KeyStroke>();
        shortcuts.put(0,KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false));
        shortcuts.put(1,KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false));
        // set checked value for ShowNodeLabels and ShowTransitionLabels       
        if (theSettings.showNodeLabels())
            viewItems[5] = "+" + viewItems[5];
        else
            viewItems[5] = "-" + viewItems[5];
        
        if (theSettings.showTransitionLabels())
            viewItems[6] = "+" + viewItems[6];
        else
            viewItems[6] = "-" + viewItems[6]; 
        setupMenu(viewItems, viewMenu, printListener, shortcuts);
        
        // The edit menu...    
        setupMenu(toolItems, toolMenu, toolListener);
        
        // The help menu...
        setupMenu(helpItems, helpMenu, printListener);
        
        add(fileMenu);
        add(editMenu);
        add(viewMenu);
        add(toolMenu);
        add(helpMenu);
    }
    
    public void updateMRUList() {
        globals.log.writeLine(Severity.Debug, "Updating MRU Menu");
        // First up, wipe the MRU menu
        recentFiles.removeAll();
        
        if (theSettings.getMRUCount() == 0) {
            globals.log.writeLine(Severity.Debug, "No MRU Items");
            return;
        }
        
        // Then re-add everything
        for (int i = 0; i < theSettings.getMRUCount(); i++) {
            if (theSettings.getMRUItem(i) != null) {
                String lItem = theSettings.getMRUItem(i);
                
                globals.log.writeLine(Severity.Debug, "Adding MRU Item: " + lItem);
                        
                JMenuItem item = new JMenuItem(lItem);
                item.setActionCommand(lItem);
                item.addActionListener(openListener);
                recentFiles.add(item);
            } else {
                globals.log.writeLine(Severity.Debug, "MRU Item NULL");
            }
        }
    }
    
    /** This constructs adds a list of strings to a JMenu using a specified actionListener
     */
    protected void setupMenu(String [ ] menuItems, JMenu theMenu, ActionListener act) {
        for (int i=0; i < menuItems.length; i++) {
            if (menuItems[i].startsWith("_")) {
                // It is a separator
                theMenu.addSeparator();
            }
            else if (menuItems[i].startsWith("+") || menuItems[i].startsWith("-")) {
                // It is a toggle menu
                JCheckBoxMenuItem item = new JCheckBoxMenuItem(menuItems[i].substring(1));
                item.addActionListener(act);
                
                if (menuItems[i].startsWith("+"))
                    item.setState(true);
                else
                    item.setState(false);                
                
                theMenu.add(item);
            }
            else {
                // Its a normal boring old menu item
                JMenuItem item = new JMenuItem(menuItems[i]);
                item.addActionListener(act);
                theMenu.add(item);
            }
        }
    }
   
    /** Does the same as the other setupMenu function except with support for shortcut
     *  keys.
     */
    protected void setupMenu(   String [ ] menuItems, 
                                JMenu theMenu, 
                                ActionListener act, 
                                HashMap<Integer, KeyStroke> shortcutKeys) {
        for (int i=0; i < menuItems.length; i++) {
            if (menuItems[i].startsWith("_")) {
                // It is a separator
                theMenu.addSeparator();
            }
            else if (menuItems[i].startsWith("+") || menuItems[i].startsWith("-")) {
                // It is a toggle menu
                JCheckBoxMenuItem item = new JCheckBoxMenuItem(menuItems[i].substring(1));
                item.addActionListener(act);
                
                if (menuItems[i].startsWith("+"))
                    item.setState(true);
                else
                    item.setState(false);
                
                // Setup the shortcut keys:
                if (shortcutKeys.containsKey(i))
                    item.setAccelerator(shortcutKeys.get(i));
                
                theMenu.add(item);
            }
            else {            
                JMenuItem item = new JMenuItem(menuItems[i]);
                item.addActionListener(act);

                // Setup the shortcut keys:
                if (shortcutKeys.containsKey(i))
                    item.setAccelerator(shortcutKeys.get(i));

                theMenu.add(item);
            }
        }
    }
    
}
