/*
    This Software (C) Copyright David Goodwin, 2007, 2008.
    
     This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * 
 * This class handles loading and saving of settings.
 */

package assignment;

import java.util.ArrayList;
import logging.*;
import logging.ILogger.Severity;
import java.io.*;

/** This class handles loading and saving of settings.
 *
 * @author David
 */
public class settings {
    
    private String SettingsFile = globals.settingsFileName;
    private ArrayList<String> MRUList = new ArrayList<String>();
       
    /* Logger Types
     * 0 DebugConsoleLogger -- Writes all messages to the console
     * 1 ConsoleLogger      -- Writes non-debug messages to the console
     * 2 NullLogger         -- Ignores all messages
     * 3 Logger             -- Writes non-debug messages to a file and the console
     * 4 DebugLogger        -- Writes all messages to a file and the console
     */
    private int LoggerType = 0;
    
    private String logFile = "log.txt";
    private String defaultDialogLocation = "";
    private String previousDialogLocation = "";
    
    private boolean showNodeLabels = true;
    private boolean showTransitionLabels = true;
    
    private boolean autogenerateNodeLabels = false;
    private boolean autogenerateTransitionLabels = false;
    
   
    public void loadSettings(String settingsFile) throws IOException {
        globals.log.writeLine("Attempting to load Settings File  \"" + SettingsFile +"\"");
        
        LTS theLTS = new LTS();
        theLTS.setFileName(SettingsFile);
        
        FileInputStream fin;		

        fin = new FileInputStream (SettingsFile);
        BufferedReader breader = new BufferedReader(new InputStreamReader(fin));            
        String curlin = breader.readLine();
        
        while (curlin != null) {
            if (curlin.startsWith("#")) {
                // Its a comment, do nothing
            } 
            else if (curlin.toLowerCase().startsWith("logtype: ")) {
                String ltype = curlin.substring(9);
                if (ltype.startsWith("DebugConsoleLogger"))
                    LoggerType = 0;
                else if (ltype.startsWith("ConsoleLogger"))
                    LoggerType = 1;
                else if (ltype.startsWith("NullLogger"))
                    LoggerType = 2;
                else if (ltype.startsWith("Logger"))
                    LoggerType = 3;
                else if (ltype.startsWith("DebugLogger"))
                    LoggerType = 4;
                else if (ltype.startsWith("DebugConsoleLogger"))
                    LoggerType = 5;
                else
                    LoggerType = 1;
            } 
            else if (curlin.toLowerCase().startsWith("logfile: ")) {
                logFile = curlin.substring(9);
            } 
            else if (curlin.toLowerCase().startsWith("defaultlocation: ")) {
                defaultDialogLocation = curlin.substring(17);
            } 
            else if (curlin.toLowerCase().startsWith("previousdirectory: ")) {
                previousDialogLocation = curlin.substring(19);
            }
            else if (curlin.toLowerCase().startsWith("shownodelabels: ")) {
                String val = curlin.toLowerCase().substring(16);
                if (val.startsWith("false"))
                    showNodeLabels = false;
                else
                    showNodeLabels = true;
            }
            else if (curlin.toLowerCase().startsWith("showtransitionlabels: ")) {
                String val = curlin.toLowerCase().substring(22);
                if (val.startsWith("false"))
                    showTransitionLabels = false;
                else
                    showTransitionLabels = true;
            }      
            else if (curlin.toLowerCase().startsWith("generatenodelabels: ")) {
                String val = curlin.toLowerCase().substring(20);
                if (val.startsWith("false"))
                    autogenerateNodeLabels = false;
                else
                    autogenerateNodeLabels = true;
            }    
            else if (curlin.toLowerCase().startsWith("generatetransitionlabels: ")) {
                String val = curlin.toLowerCase().substring(26);
                if (val.startsWith("false"))
                    autogenerateTransitionLabels = false;
                else
                    autogenerateTransitionLabels = true;
            }             
            else if (curlin.startsWith("BEGIN MRU:")) {
                curlin = breader.readLine();
                while (!curlin.startsWith(":END MRU") && curlin != null) {
                    MRUList.add(curlin);
                    curlin = breader.readLine();
                }
            }
           
            curlin = breader.readLine();
        }
        fin.close();
    }
    
    public void saveSettings() {
        FileOutputStream fout;		

        try {
            fout = new FileOutputStream (SettingsFile);
            PrintStream ps = new PrintStream(fout);

            /* Write out the Logger selection */
            String ltype = "ConsoleLogger";
            switch (getLoggerType()) {
                case 0:
                    ltype =  "DebugConsoleLogger";
                case 1:
                    ltype = "ConsoleLogger";
                case 2:
                    ltype = "NullLogger";
                case 3:
                    ltype = "Logger";
                case 4:
                    ltype = "DebugLogger";
                default:
                    ltype = "DebugConsoleLogger";
            }
            ps.println("LogType: " + ltype);

            /* Write out the location of the log file */
            ps.println("LogFile: " + logFile);
            
            /* Where should the open dialog end up by default? here of course. */
            ps.println("DefaultLocation: " + defaultDialogLocation);
            
            /* What was our previous directory? */
            ps.println("PreviousDirectory: " + previousDialogLocation);
            
            /* Save label drawing preferences */
            if (showNodeLabels)
                ps.println("ShowNodeLabels: True");
            else
                ps.println("ShowNodeLabels: False");
            
            if (showTransitionLabels)
                ps.println("ShowTransitionLabels: True");
            else
                ps.println("ShowTransitionLabels: False");
                       
            if (autogenerateNodeLabels)
                ps.println("GenerateNodeLabels: True");
            else
                ps.println("GenerateNodeLabels: False");
            
            if (autogenerateTransitionLabels)
                ps.println("GenerateTransitionLabels: True");
            else
                ps.println("GenerateTransitionLabels: False");         
            
            /* Write out the MRU list */
            ps.println("BEGIN MRU:");
            for (int i = 0; i < MRUList.size(); i++) {
                ps.println(MRUList.get(i));
            }
            ps.println(":END MRU");

            fout.close();	
        } catch(Exception e) {
            globals.log.writeLine(Severity.Error, "Failed to open settings file");
        }
    }    
    
    /** Gets an item for the Most-Recently-Used-File list
     * 
     * @param index The item to get
     * @return The item specified
     */
    public String getMRUItem(int index) {
        if (MRUList.size() == 0) {
            globals.log.writeLine(Severity.Debug, "MRU List Empty. Nothing to retrieve.");
            return null;
        }
        else if (MRUList.size() > index )
            return MRUList.get(index);
        else {
            globals.log.writeLine(Severity.Debug, "MRU Index Not Present: " + Integer.toString(index));
            return null;
        }
    }
    
    /** Adds the specified item to the MRU list.
     * 
     * @param item The Item to add.
     */
    public void addMRUItem(String item) {
        if (MRUList.size() == 0) {
            MRUList.add(0, item);
        }
        else if (MRUList.contains(item)) {
            MRUList.remove(item);
            MRUList.add(0, item);
        }
        else if (MRUList.size() < globals.MRUSize) {
            MRUList.add(0, item);
        } else {
            MRUList.remove(MRUList.size() - 1);            
            MRUList.add(0,item);
        }
    }
    
    public int getMRUCount() {
        return MRUList.size();
    }
    
    public void clearMRU() {
        MRUList.clear();
    }
    
    public int getLoggerType() {
        return LoggerType;
    }
    
    public void setLoggerType(int loggerType) {
        LoggerType = loggerType;
    }
    
    public String getLogFile() {
        return logFile;
    }
    
    public void setLogFile(String filename) {
        logFile = filename;
    }
    
    public void setLastDirectory(String directory) {
        previousDialogLocation = directory;
    }
    
    public String getDialogLocation() {
        if (defaultDialogLocation.isEmpty())
            if (previousDialogLocation.isEmpty())
                return null;
            else
                return previousDialogLocation;
        else
            return defaultDialogLocation;
    }
    
    public boolean showNodeLabels() {
        return showNodeLabels;
    }
    
    public boolean showTransitionLabels() {
        return showTransitionLabels;
    }
    
    public boolean autoGenerateNodeLabels() {
        return autogenerateNodeLabels;
    }
    
    public boolean autoGenerateTransitionLabels() {
        return autogenerateTransitionLabels;
    }
    
    public void showNodeLabels(boolean value) {
        showNodeLabels = value;
    }
    
    public void showTransitionLabels(boolean value) {
        showTransitionLabels = value;
    }     
    
    public void autoGenerateNodeLabels(boolean value) {
        autogenerateNodeLabels = value;
    }
    
    public void autoGenerateTransitionLabels(boolean value) {
        autogenerateTransitionLabels = value;
    }
}
